==> ignorance-1.3.0.1.tar.bz2: ignorance-1.3.0.1/ignorance_level.c <==
#include "ignorance_level.h"
#include <string.h>
#include <strings.h>

//void	ignorance_g_string_free(gpointer user_data);
//void ignorance_g_string_free_ghashtable(gpointer ght);

gboolean	assign_level_token(ignorance_level *lvl,const gchar *tokentxt);
gint	g_string_compare(gconstpointer a, gconstpointer z);

ignorance_level*	ignorance_level_new(){
	ignorance_level *il=(ignorance_level*)g_malloc(sizeof(ignorance_level));
	il->index=IGNORANCE_LEVEL_DEFAULT;
	il->allow_passthrough=FALSE;
	il->name=g_string_new("Default");
	il->denizens=NULL;
	il->rules=g_ptr_array_new();
	il->upper_threshhold=9;
	il->lower_threshhold=-9;

	return il;
}//ignorance_level_new



void	ignorance_level_free(ignorance_level *il){
	g_string_free(il->name,TRUE);
	g_list_free(il->denizens);
	g_ptr_array_foreach(il->rules,ignorance_rule_free_g,NULL);
}//ignorance_level_free



void	ignorance_level_free_g(gpointer il,gpointer user_data){
	ignorance_level_free((ignorance_level*)il);
}//ignorance_level_free_g


gboolean	ignorance_level_add_rule(ignorance_level *level,ignorance_rule *rule){
	g_ptr_array_add(level->rules,(gpointer)rule);
	return TRUE;
}//ignorance_level_add_rule


ignorance_rule*	ignorance_level_get_rule(ignorance_level *level,const GString *rulename){
	int i=0;
	ignorance_rule *rule=NULL;
	
	for(i=0;i<level->rules->len;++i){
		rule=g_ptr_array_index(level->rules,i);
		if(g_string_equal(rulename,rule->name)){
			return g_ptr_array_index(level->rules,i);
		}//if
	}//for

	return NULL;
}//ignorance_level_get_rule


gboolean	ignorance_level_remove_rule(ignorance_level *level,const GString *rulename){
	return g_ptr_array_remove_fast(level->rules,ignorance_level_get_rule(level,rulename));
}//ignorance_level_remove_rule



gboolean	ignorance_level_add_denizen(ignorance_level *level, const GString *username){
	if(ignorance_level_has_denizen(level,username)){
		return FALSE;
	}//if
	level->denizens=g_list_append(level->denizens,(gpointer)g_string_new(username->str));
	return TRUE;
}//ignorance_level_add_denizen

gboolean	ignorance_level_has_denizen_regex(ignorance_level	*level,const gchar *regex, GList	**denizens){
	regex_t	*reg=(regex_t*)g_malloc(sizeof(regex_t));
	gboolean	rv=FALSE;
	GString	*denizen;
	GList	*cursor;
	 
	if(regcomp(reg,regex,REG_EXTENDED | REG_NOSUB)){
		//fprintf(stderr,"Ignorance: Error parsing regex %s\n",regex);
		gaim_debug(GAIM_DEBUG_ERROR, "ignorance", "Error parsing regex %s\n", regex);
		return FALSE;
	}//if
	
	for(cursor=g_list_first(level->denizens);cursor;cursor=g_list_next(cursor)){
		denizen=(GString*)(cursor->data);
		if(!(regexec(reg,denizen->str,1,NULL,0))){
			(*denizens)=g_list_append(*denizens,(gpointer)g_string_new(denizen->str));
			rv=TRUE;
		}//match
	}//for
	
	g_free(reg);
	return rv;
}//ignorance_level_has_denizen_regex


gboolean	ignorance_level_has_denizen(ignorance_level *level,const GString *username){
	//fprintf(stderr,"Ignorance: Preparing to find %s in %s: %d\n",username->str,level->name->str,g_list_position(level->denizens,g_list_last(level->denizens)));
	return (NULL!=g_list_find_custom(level->denizens,(gconstpointer)username,g_string_compare));
}//ignorance_level_has_denizen



//returns the difference of two GStrings
gint	g_string_compare(gconstpointer a, gconstpointer z){
	const GString 	*gsa=(const GString*)a, 
					*gsz=(const GString*)z;
	//fprintf(stderr,"Ignorance: Comparing %s to %s\n",gsa->str,gsz->str);
	return g_ascii_strcasecmp(gsa->str, gsz->str);
}//g_string_compare


gboolean	ignorance_level_remove_denizen(ignorance_level *level, const GString *username){
	GList *den=g_list_find_custom(level->denizens,(gconstpointer)username,g_string_compare);
	
	if(den){
		level->denizens=g_list_delete_link(level->denizens,den);
		//fprintf(stderr,"Ignorance: Removed %s from %s\n",username->str,level->name->str);
	}//if
	else{
		//fprintf(stderr,"Ignorance: %s not found in %s\n",username->str,level->name->str);
	}//not found
	
	return (NULL!=den);
}//ignorance_level_remove_denizen


gint	ignorance_level_rulecheck(ignorance_level *level,const GString *text, gint flags){
	int	i=0;
	int	totalscore=0;
	
	//fprintf(stderr,"Ignorance: preparing to check %s against rules for %s\n",text->str,level->name->str);

	for(i=0;i<level->rules->len;++i){
		totalscore+=ignorance_rule_rulecheck(((ignorance_rule*)g_ptr_array_index(level->rules,i)),text, flags);
	}//for

	return totalscore;
}//ignorance_level_rulecheck

ignorance_level*	ignorance_level_read(const gchar *lvltext){
	gchar	*tokptr=strtok((gchar*)lvltext," ");
	ignorance_level *lvl=ignorance_level_new();

	if(!tokptr){
		ignorance_level_free(lvl);
		return NULL;
	}//if


	for(tokptr=strtok(NULL," ");tokptr;tokptr=strtok(NULL," ")){
		assign_level_token(lvl,tokptr);
	}//for

	return lvl;
}//ignorance_level_read

// Parses out a token of the form tokenname="value" and assigns it to a rulename
// 
// rule is the rule to be updated
// tokentxt is the token string
// true returned if token is valid and successfully added to the rule
//
// level name1="value1" name2="value2" ...
gboolean	assign_level_token(ignorance_level *lvl,const gchar *tokentxt){
	gchar	*name=NULL,
		*value=NULL;
	gboolean rv=TRUE;
	gint	cursor=0;

		value=strchr(tokentxt,'=');
		if(value){	
			(*value)='\0';
			++value;

			//strip "value" quotes
			if('"'==(*value)){
				++value;
				cursor=strlen(value)-1;
				if('"'==value[cursor]){
					value[cursor]='\0';
				}//if
			}//if
			name=(gchar*)tokentxt;

			//HACK!
			if(!strncasecmp(name,"name",BUFSIZ)){
				g_string_assign(lvl->name,value);
			}else if(!strncasecmp(name,"index",BUFSIZ)){
				lvl->index=atoi(value);
			}else if(!strncasecmp(name,"upper_threshhold",BUFSIZ)){
				lvl->upper_threshhold=atoi(value);
			}else if(!strncasecmp(name,"lower_threshhold",BUFSIZ)){
				lvl->lower_threshhold=atoi(value);
			}else if(!strncasecmp(name,"allow_passthrough",BUFSIZ)){
				lvl->allow_passthrough=atoi(value);
			}else{
				rv=FALSE;
			}//long, ugly switch
		}else{
			rv=FALSE;
		}//value

		return rv;
}//assign_level_token


gboolean	ignorance_level_write(ignorance_level *level,FILE *f){
	gint i;
	GList	*cursor=NULL;
	GString	*denizen=NULL;
	
	fprintf(f,"level index=\"%d\" name=\"%s\" upper_threshhold=\"%d\" lower_threshhold=\"%d\" allow_passthrough=\"%d\"\n",
				level->index,level->name->str,level->upper_threshhold,level->lower_threshhold, level->allow_passthrough);
	
	for(i=0;i<level->rules->len;++i){
		ignorance_rule_write(g_ptr_array_index(level->rules,i),f);
	}//for
	
	for(cursor=g_list_first(level->denizens);cursor;cursor=g_list_next(cursor)){
		denizen=(GString*)(cursor->data);
		fprintf(f,"%s\n",denizen->str);
	}//for
	
	return TRUE;
}//ignorance_level_write

==> ignorance-2.2.tar.bz2: ignorance-2.2/ignorance_level.c <==
#include "ignorance_level.h"
#include "ignorance.h"
#include "ignorance_violation.h"

#include <string.h>
#include <strings.h>


gboolean	assign_level_token(ignorance_level *lvl,const gchar *tokentxt);
gint	g_string_compare(gconstpointer a, gconstpointer z);

ignorance_level*	ignorance_level_new(){
	ignorance_level *il=(ignorance_level*)g_malloc(sizeof(ignorance_level));
	il->name=g_string_new("Default");
	il->denizens=NULL;
	il->rules=g_ptr_array_new();

	return il;
}//ignorance_level_new



void	ignorance_level_free(ignorance_level *il){
	g_string_free(il->name,TRUE);
	g_list_free(il->denizens);
	g_ptr_array_foreach(il->rules,ignorance_rule_free_g,NULL);
}//ignorance_level_free



void	ignorance_level_free_g(gpointer il,gpointer user_data){
	ignorance_level_free((ignorance_level*)il);
}//ignorance_level_free_g


gboolean	ignorance_level_add_rule(ignorance_level *level,ignorance_rule *rule){
	g_ptr_array_add(level->rules,(gpointer)rule);
	return TRUE;
}//ignorance_level_add_rule


ignorance_rule*	ignorance_level_get_rule(ignorance_level *level,const GString *rulename){
	int i=0;
	ignorance_rule *rule=NULL;
	
	for(i=0;i<level->rules->len;++i){
		rule=g_ptr_array_index(level->rules,i);
		if(g_string_equal(rulename,rule->name)){
			return g_ptr_array_index(level->rules,i);
		}//if
	}//for

	return NULL;
}//ignorance_level_get_rule


gboolean	ignorance_level_remove_rule(ignorance_level *level,const GString *rulename){
	return g_ptr_array_remove_fast(level->rules,ignorance_level_get_rule(level,rulename));
}//ignorance_level_remove_rule


gboolean	ignorance_level_add_denizen(ignorance_level *level, const GString *username){
	if(ignorance_level_has_denizen(level,username)){
		return FALSE;
	}//if
	level->denizens=g_list_prepend(level->denizens,(gpointer)g_string_new(username->str));
	return TRUE;
}//ignorance_level_add_denizen


gboolean	ignorance_level_add_denizen_fast(ignorance_level *level, const GString *username){
	level->denizens=g_list_prepend(level->denizens,(gpointer)g_string_new(username->str));
	return TRUE;
}//ignorance_level_add_denizen


#ifdef HAVE_REGEX_H
gboolean	ignorance_level_has_denizen_regex(ignorance_level	*level,const gchar *regex, GList	**denizens){
	regex_t reg;
	//regex_t	*reg=(regex_t*)g_malloc(sizeof(regex_t));
	gboolean	rv=FALSE;
	GString	*denizen;
	GList	*cursor;
	 
	if(regcomp(&reg,regex,REG_EXTENDED | REG_NOSUB)){
		gaim_debug(GAIM_DEBUG_ERROR, "ignorance", "Error parsing regex %s\n", regex);
		regfree(&reg);
		return FALSE;
	}//if
	
	for(cursor=g_list_first(level->denizens);cursor;cursor=g_list_next(cursor)){
		denizen=(GString*)(cursor->data);
		if(!(regexec(&reg,denizen->str,1,NULL,0))){
			(*denizens)=g_list_prepend(*denizens,(gpointer)g_string_new(denizen->str));
			rv=TRUE;
		}//match
	}//for
	
	regfree(&reg);
	//g_free(reg);
	return rv;
}//ignorance_level_has_denizen_regex
#endif


gboolean	ignorance_level_has_denizen(ignorance_level *level,const GString *username){
	return (NULL!=g_list_find_custom(level->denizens,(gconstpointer)username,g_string_compare));
}//ignorance_level_has_denizen


//returns the difference of two GStrings
gint	g_string_compare(gconstpointer a, gconstpointer z){
	const GString 	*gsa=(const GString*)a, 
					*gsz=(const GString*)z;

	return g_ascii_strcasecmp(gsa->str, gsz->str);
}//g_string_compare


gboolean	ignorance_level_remove_denizen(ignorance_level *level, const GString *username){
	GList *den=g_list_find_custom(level->denizens,(gconstpointer)username,g_string_compare);
	
	if(den){
		level->denizens=g_list_delete_link(level->denizens,den);
	}//if
	
	return (NULL!=den);
}//ignorance_level_remove_denizen


gint	ignorance_level_rulecheck(ignorance_level *level,const GString *username,const GString *text, gint flags,GList **violations){
	int	i=0,
		totalscore=0,
		curscore;
	ignorance_rule *cur;

	for(i=0;i<level->rules->len;++i){
		cur=(ignorance_rule*)g_ptr_array_index(level->rules,i);
		if(cur->flags & IGNORANCE_APPLY_USER){
			curscore=ignorance_rule_rulecheck(cur,username,flags);
			totalscore|=curscore;
			if(curscore){
				if(curscore & IGNORANCE_FLAG_MESSAGE){
					(*violations)=g_list_prepend(*violations,ignorance_violation_newp(IGNORANCE_FLAG_MESSAGE,cur->message));
				}//message
				if(curscore & IGNORANCE_FLAG_SOUND){
					(*violations)=g_list_prepend(*violations,ignorance_violation_newp(IGNORANCE_FLAG_SOUND,cur->sound));
				}//sound
				if(curscore & IGNORANCE_FLAG_EXECUTE){
					(*violations)=g_list_prepend(*violations,ignorance_violation_newp(IGNORANCE_FLAG_EXECUTE,cur->command));
				}//command
			}//get violations
		}//if
		curscore=ignorance_rule_rulecheck(cur,text, flags & (~IGNORANCE_APPLY_USER));
		totalscore|=curscore;
		if(curscore){
			if(curscore & IGNORANCE_FLAG_MESSAGE){
				(*violations)=g_list_prepend(*violations,ignorance_violation_newp(IGNORANCE_FLAG_MESSAGE,cur->message));
			}//message
			if(curscore & IGNORANCE_FLAG_SOUND){
				(*violations)=g_list_prepend(*violations,ignorance_violation_newp(IGNORANCE_FLAG_SOUND,cur->sound));
			}//sound
			if(curscore & IGNORANCE_FLAG_EXECUTE){
				(*violations)=g_list_prepend(*violations,ignorance_violation_newp(IGNORANCE_FLAG_EXECUTE,cur->command));
			}//command
		}//get violations
	}//for

	return totalscore;
}//ignorance_level_rulecheck


ignorance_level*	ignorance_level_read_old(const gchar *lvltext){
	gchar	*tokptr=strchr((gchar*)lvltext,' '),
			**tokens=NULL;
	ignorance_level *lvl=ignorance_level_new();
	int i=0;

	if(!tokptr){
		ignorance_level_free(lvl);
		return NULL;
	}//if

	tokens=g_strsplit(lvltext," ",INT_MAX);

	for(i=0;tokens[i];++i){
		assign_level_token(lvl,tokens[i]);
	}//for

	g_strfreev(tokens);

	return lvl;
}//ignorance_level_read_old


ignorance_level*	ignorance_level_read(const gchar *lvltext){
	gchar	*tokptr=strchr((gchar*)lvltext,'\n'),
			**tokens=NULL;
	ignorance_level *lvl=ignorance_level_new();
	int i=0;

	if(!tokptr){
		ignorance_level_free(lvl);
		return ignorance_level_read_old(lvltext);
	}//if

	tokens=g_strsplit(lvltext,"\n",INT_MAX);


	for(i=0;tokens[i];++i){
		assign_level_token(lvl,tokens[i]);
	}//for

	g_strfreev(tokens);

	return lvl;
}//ignorance_level_read


// Parses out a token of the form tokenname="value" and assigns it to a rulename
// 
// rule is the rule to be updated
// tokentxt is the token string
// true returned if token is valid and successfully added to the rule
//
// level name1="value1" name2="value2" ...
gboolean	assign_level_token(ignorance_level *lvl,const gchar *tokentxt){
	gchar	*name=NULL,
		*value=NULL;
	gboolean rv=TRUE;
	gint	cursor=0;

		value=strchr(tokentxt,'=');
		if(value){	
			(*value)='\0';
			++value;

			//strip "value" quotes
			if('"'==(*value)){
				++value;
				cursor=strlen(value)-1;
				if('"'==value[cursor]){
					value[cursor]='\0';
				}//if
			}//if
			name=(gchar*)tokentxt;

			//HACK!
			if(!strncasecmp(name,"name",BUFSIZ)){
				g_string_assign(lvl->name,value);
			}else{
				rv=FALSE;
			}//long, ugly switch - now shortened!
		}else{
			rv=FALSE;
		}//value

		return rv;
}//assign_level_token


gboolean	ignorance_level_write(ignorance_level *level,FILE *f){
	gint i;
	GList	*cursor=NULL;
	GString	*denizen=NULL;
	
	fprintf(f,"level\nname=\"%s\"\n/level\n",
				level->name->str);
	
	for(i=0;i<level->rules->len;++i){
		ignorance_rule_write(g_ptr_array_index(level->rules,i),f);
	}//for
	
	for(cursor=g_list_first(level->denizens);cursor;cursor=g_list_next(cursor)){
		denizen=(GString*)(cursor->data);
		fprintf(f,"%s\n",denizen->str);
	}//for
	
	return TRUE;
}//ignorance_level_write
